searchData={"items":[{"type":"module","title":"Lua","doc":"`Lua` is an ergonomic interface to [Luerl](https://github.com/rvirding/luerl), aiming to be the best way to use Luerl from Elixir.","ref":"Lua.html"},{"type":"module","title":"Features - Lua","doc":"* Ergonomic API for Elixir <> Lua FFI\n* Improved error messages\n* Deep-setting variables and state\n* Excellent documentation and guides for working with Luerl\n\n> ##","ref":"Lua.html#module-features"},{"type":"module","title":"Lua the Elixir library vs Lua the language {: .info} - Lua","doc":"> When referring to this library, `Lua` will be stylized as a link.\n> \n> References to Lua the language will be in plaintext and not linked.","ref":"Lua.html#module-lua-the-elixir-library-vs-lua-the-language-info"},{"type":"module","title":"Executing Lua - Lua","doc":"`Lua` can be run using the `eval!/2` function\n\n``` elixir\n    iex> {[4], _} =\n    ...>   Lua.eval!(\"\"\"\n    ...>   return 2 + 2\n    ...>   \"\"\")\n\n```","ref":"Lua.html#module-executing-lua"},{"type":"module","title":"Exposing Elixir functions to Lua - Lua","doc":"`Lua` provides the `deflua` macro for exposing Elixir functions to Lua\n\n``` elixir\ndefmodule MyAPI do\n  use Lua.API\n      \n  deflua double(v), do: 2 * v\nend\n    \nlua = Lua.new() |> Lua.load_api(MyAPI)\n\n{[10], _} =\n  Lua.eval!(lua, \"\"\"\n  return double(5)\n  \"\"\")\n\n```","ref":"Lua.html#module-exposing-elixir-functions-to-lua"},{"type":"module","title":"Calling Lua functions from Elixir - Lua","doc":"`Lua` can be used to expose complex functions written in Elixir. In some cases, you may want to call Lua functions from Elixir. This can\nbe achieved with the `Lua.call_function!/3` function\n\n``` elixir\ndefmodule MyAPI do\n  use Lua.API, scope: \"example\"\n\n  deflua foo(value), state do\n    Lua.call_function!(state, [:string, :lower], [value])\n  end\nend\n\nlua = Lua.new() |> Lua.load_api(MyAPI)\n\n{[\"wow\"], _} = Lua.eval!(lua, \"return example.foo(\\\"WOW\\\")\")\n```","ref":"Lua.html#module-calling-lua-functions-from-elixir"},{"type":"module","title":"Modify Lua state from Elixir - Lua","doc":"You can also use `Lua` to modify the state of the lua environment inside your Elixir code. Imagine you have a queue module that you\nwant to implement in Elixir, with the queue stored in a global variable\n\n``` elixir\ndefmodule Queue do\n  use Lua.API, scope: \"q\"\n  \n  deflua push(v), state do\n    # Pull out the global variable \"my_queue\" from lua\n    queue = Lua.get!(state, [:my_queue])\n    \n    # Call the Lua function table.insert(table, value)\n    {[], state} = Lua.call_function!(state, [:table, :insert], [queue, v])\n    \n    # Return the modified lua state with no return values\n    {[], state}\n  end\nend\n\nlua = Lua.new() |> Lua.load_api(Queue)\n\n{[queue], _} =\n  Lua.eval!(lua, \"\"\"\n  my_queue = {}\n\n  q.push(\"first\")\n  q.push(\"second\")\n\n  return my_queue\n  \"\"\")\n  \n[\"first\", \"second\"] = Lua.Table.as_list(queue)\n```","ref":"Lua.html#module-modify-lua-state-from-elixir"},{"type":"module","title":"Credits - Lua","doc":"`Lua` piggy-backs off of Robert Virding's [Luerl](https://github.com/rvirding/luerl) project, which implements a Lua lexer, parser, and full-blown Lua virtual machine that runs inside the BEAM.","ref":"Lua.html#module-credits"},{"type":"function","title":"Lua.call_function!/3","doc":"Calls a function in Lua's state\n\n    iex> {[ret], _lua} = Lua.call_function!(Lua.new(), [:string, :lower], [\"HELLO ROBERT\"])\n    iex> ret\n    \"hello robert\"\n\nReferences to functions can also be passed\n\n    iex> {[ref], lua} = Lua.eval!(\"return string.lower\")\n    iex> {[ret], _lua} = Lua.call_function!(lua, ref, [\"FUNCTION REF\"])\n    iex> ret\n    \"function ref\"\n\nThis is also useful for executing Lua function's inside of Elixir APIs\n\n```elixir\ndefmodule MyAPI do\n  use Lua.API, scope: \"example\"\n\n  deflua foo(value), state do\n    Lua.call_function!(state, [:string, :lower], [value])\n  end\nend\n\nlua = Lua.new() |> Lua.load_api(MyAPI)\n\n{[\"wow\"], _} = Lua.eval!(lua, \"return example.foo(\"WOW\")\")\n```","ref":"Lua.html#call_function!/3"},{"type":"function","title":"Lua.decode!/2","doc":"Decodes a Lua value from its internal form\n\n    iex> {encoded, lua} = Lua.encode!(Lua.new(), %{a: 1})\n    iex> Lua.decode!(lua, encoded)\n    [{\"a\", 1}]","ref":"Lua.html#decode!/2"},{"type":"function","title":"Lua.encode!/2","doc":"Encodes a Lua value into its internal form\n\n    iex> {encoded, _} = Lua.encode!(Lua.new(), %{a: 1})\n    iex> encoded\n    {:tref, 14}","ref":"Lua.html#encode!/2"},{"type":"function","title":"Lua.eval!/2","doc":"Evaluates the Lua script, returning any returned values and the updated\nLua environment\n\n    iex> {[42], _} = Lua.eval!(Lua.new(), \"return 42\")","ref":"Lua.html#eval!/2"},{"type":"function","title":"Lua.get!/2","doc":"Gets a table value in Lua\n\n    iex> state = Lua.set!(Lua.new(), [:hello], \"world\")\n    iex> Lua.get!(state, [:hello])\n    \"world\"\n\nWhen a value doesn't exist, it returns nil\n\n    iex> Lua.get!(Lua.new(), [:nope])\n    nil\n\nIt can also get nested values\n\n    iex> state = Lua.set!(Lua.new(), [:a, :b, :c], \"nested\")\n    iex> Lua.get!(state, [:a, :b, :c])\n    \"nested\"","ref":"Lua.html#get!/2"},{"type":"function","title":"Lua.load_api/3","doc":"Inject functions written with the `deflua` macro into the Lua runtime\n\nSimilar to `:luerl_new.load_module/3` but without the `install` callback","ref":"Lua.html#load_api/3"},{"type":"function","title":"Lua.load_file!/2","doc":"Loads a Lua file into the environment. Any values returned in the global\nscope are thrown away.\n\nMimics the functionality of Lua's [dofile](https://www.lua.org/manual/5.4/manual.html#pdf-dofile)","ref":"Lua.html#load_file!/2"},{"type":"function","title":"Lua.new/1","doc":"Initializes a Lua VM sandbox\n\n    iex> Lua.new()\n\nBy default, potentially dangerous functions are sandboxed. To disable, use the `sandboxed` option\n\n    iex> Lua.new(sandboxed: [])\n\nAlternatively, you can pass your own list of functions to sandbox. This is equivalent to calling\n`Lua.sandbox/2`.\n\n    iex> Lua.new(sandboxed: [[:os, :exit]])","ref":"Lua.html#new/1"},{"type":"function","title":"Options - Lua.new/1","doc":"* `:sandboxed` - list of paths to be sandboxed, e.g. `sandboxed: [[:require], [:os, :exit]]`\n* `:exclude` - list of paths to exclude from the sandbox, e.g. `exclude: [[:require], [:package]]`","ref":"Lua.html#new/1-options"},{"type":"function","title":"Lua.sandbox/2","doc":"Sandboxes the given path, swapping out the implementation with\na function that raises when called\n\n    iex> lua = Lua.new(sandboxed: [])\n    iex> Lua.sandbox(lua, [:os, :exit])","ref":"Lua.html#sandbox/2"},{"type":"function","title":"Lua.set!/3","doc":"Sets a table value in Lua. Nested keys will create\nintermediate tables\n\n    iex> Lua.set!(Lua.new(), [:hello], \"World\")\n\nIt can also set nested values\n\n    iex> Lua.set!(Lua.new(), [:a, :b, :c], [])\n\nThese table values are availble in lua scripts\n\n    iex> lua = Lua.set!(Lua.new(), [:a, :b, :c], \"nested!\")\n    iex> {result, _} = Lua.eval!(lua, \"return a.b.c\")\n    iex> result\n    [\"nested!\"]","ref":"Lua.html#set!/3"},{"type":"function","title":"Lua.set_lua_paths/2","doc":"Sets the path patterns that Luerl will look in when requiring Lua scripts. For example,\nif you store Lua files in your application's priv directory:\n\n    iex> lua = Lua.new(exclude: [[:package], [:require]])\n    iex> Lua.set_lua_paths(lua, [\"myapp/priv/lua/?.lua\", \"myapp/lua/?/init.lua\"])\n\nNow you can use the [Lua require](https://www.lua.org/pil/8.1.html) function to import\nthese scripts\n\n> ##","ref":"Lua.html#set_lua_paths/2"},{"type":"function","title":"Warning {: .warning} - Lua.set_lua_paths/2","doc":"> In order to use `Lua.set_lua_paths/2`, the following functions cannot be sandboxed:\n> * `[:package]`\n> * `[:require]`\n>\n> By default these are sandboxed, see the `:exclude` option in `Lua.new/1` to allow them.","ref":"Lua.html#set_lua_paths/2-warning-warning"},{"type":"behaviour","title":"Lua.API","doc":"Defines the Behaviour for defining a Lua API\n\nTo create a module that exports functions to the global scope\n\n    defmodule MyAPI do\n      use Lua.API\n\n      # Can be called via `print(\"hi\")` in lua\n      deflua print(msg), do: IO.puts msg\n    end\n\nOptionally, you can provide a scope\n\n    defmodule SpecificAPI do\n      use Lua.API, scope: \"namespace.domain\"\n\n      # Can be called via `namespace.domain.foo(5)` in lua\n      deflua foo(v), do: v\n    end\n\nYou can access Lua state\n\n    defmodule State do\n      use Lua.API\n\n      deflua bar(name), state do\n        # Pull's the value of `number` out of state\n        val = Lua.get!(state, [:number])\n\n        2 * val\n      end\n    end\n\nRegular functions are not exported\n\n    defmodule SpecificAPI do\n      use Lua.API\n\n      # Won't be exposed\n      def baz(v), do: v\n    end","ref":"Lua.API.html"},{"type":"macro","title":"Lua.API.deflua/2","doc":"","ref":"Lua.API.html#deflua/2"},{"type":"macro","title":"Lua.API.deflua/3","doc":"Define a function that can be exposed in Lua","ref":"Lua.API.html#deflua/3"},{"type":"macro","title":"Lua.API.runtime_exception!/1","doc":"Raises a runtime exception inside an API function, displaying contextual\ninformation about where the exception was raised.","ref":"Lua.API.html#runtime_exception!/1"},{"type":"callback","title":"Lua.API.scope/0","doc":"","ref":"Lua.API.html#c:scope/0"},{"type":"module","title":"Lua.Table","doc":"In Lua, [tables](https://www.lua.org/pil/2.5.html) are the fundamental datastructure,\nwhich are used both as associative arrays (maps), and arrays (lists).\n\n`Lua.Table` provides some utilities for working with Lua tables when passed back to\nElixir.","ref":"Lua.Table.html"},{"type":"function","title":"Lua.Table.as_list/2","doc":"Converts a Lua table into a list. Assumes that the\ntable is correctly ordered.\n\n    iex> Lua.Table.as_list([{1, \"a\"}, {2, \"b\"}, {3, \"c\"}])\n    [\"a\", \"b\", \"c\"]\n\nTo ensure the list is ordered, you can pass the `:sort` option\n\n    iex> Lua.Table.as_list([{2, \"b\"}, {1, \"a\"}, {3, \"c\"}])\n    [\"b\", \"a\", \"c\"]\n\n    iex> Lua.Table.as_list([{2, \"b\"}, {1, \"a\"}, {3, \"c\"}], sort: true)\n    [\"a\", \"b\", \"c\"]","ref":"Lua.Table.html#as_list/2"},{"type":"function","title":"Lua.Table.as_map/1","doc":"Converts a Lua table into a map\n\n    iex> Lua.Table.as_map([{\"a\", 1}, {\"b\", 2}])\n    %{\"a\" => 1, \"b\" => 2}","ref":"Lua.Table.html#as_map/1"},{"type":"function","title":"Lua.Table.deep_cast/1","doc":"Converts a Lua table into more \"native\" feeling lists and\nmaps, deeply traversing any sub-tables.\n\nIt uses the heuristic that maps with integer keys starting\nas 1 will be auto-cast into lists\n\n    iex> Lua.Table.deep_cast([{\"a\", 1}, {\"b\", [{1, 3}, {2, 4}]}])\n    %{\"a\" => 1, \"b\" => [3, 4]}","ref":"Lua.Table.html#deep_cast/1"},{"type":"exception","title":"Lua.CompilerException","doc":"","ref":"Lua.CompilerException.html"},{"type":"exception","title":"Lua.RuntimeException","doc":"","ref":"Lua.RuntimeException.html"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,49,46,50]}}